package com.thalesgroup.cheddar.MARTE2cheddar.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.thalesgroup.cheddar.MARTE2cheddar.Activator;
import com.thalesgroup.cheddar.MARTE2cheddar.MARTE2CheddarXML;
import com.thalesgroup.java.log.Log;


/**
 * 
 * <copyright>
 * Thales MARTE to Cheddar (Copyright (c) THALES 2007 All rights reserved) is free software; you can redistribute itand/or modify
 * it under the terms of the Eclipse Public License as published in http://www.eclipse.org/legal/epl-v10.html
 *
 * Thales MARTE to Cheddar is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License for more details.
 * </copyright>
 * 
 * @author Nicolas Vienne
 *
 */
public class LaunchCheck implements IWorkbenchWindowActionDelegate {
	
	protected IFile infile = null;
	/**
	 * The constructor.
	 */
	public LaunchCheck() {
		
	}

	/**
	 * The action has been activated. The argument of the
	 * method represents the 'real' action sitting
	 * in the workbench UI.
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {
		if(action.getId().equals("MARTE2Cheddar.CleanModelErrors")|| action.getId().equals("com.thalesgroup.cheddar.MARTE2cheddar.CheddarClean")) {
			if(infile != null) {
				try {
					MARTE2CheddarXML m2x = MARTE2CheddarXML.getInstance();
					m2x.cleanErrors(infile);
				} catch (Exception exception) {
					Log.errorMessage(Activator.PLUGIN_ID,"Error while cleaning model errors",exception); 
				}
			}
		} else if (action.getId().equals("MARTE2Cheddar.CheckModel") || action.getId().equals("com.thalesgroup.cheddar.MARTE2cheddar.CheddarCheckModelAction")) {
			if(infile != null) {
				try {
					MARTE2CheddarXML m2x = MARTE2CheddarXML.getInstance();
					m2x.initCheck(infile);
					new ProgressMonitorDialog(new Shell()).run(true, false, m2x);
				} catch (Exception exception) {
					Log.errorMessage(Activator.PLUGIN_ID,"Error while checking model", exception);
				}
			}
		}
	}

	/**
	 * Selection in the workbench has been changed. We 
	 * can change the state of the 'real' action here
	 * if we want, but this can only happen after 
	 * the delegate has been created.
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof ITreeSelection){
			ITreeSelection treeSelection = (ITreeSelection) selection;
			Object firstElement = treeSelection.getFirstElement();
			if (firstElement instanceof IFile){
				infile = (IFile) firstElement;
			}
		}
	}

	/**
	 * We can use this method to dispose of any system
	 * resources we previously allocated.
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
	}

	/**
	 * We will cache window object in order to
	 * be able to provide parent shell for the message dialog.
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
	}
}