/**
 * <copyright>
 * Thales MARTE To Cheddar(Copyright (c) THALES 2007 All rights reserved)
 * is free software; you can redistribute it and/or modify
 * it under the terms of the Eclipse Public License as published
 * in http://www.eclipse.org/legal/epl-v10.html
 * Thales MARTE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Eclipse Public License for more details.
 * </copyright>
 *
 * $Id: CheddarSwitch.java,v 1.1 2008/01/23 16:59:01 emaes Exp $
 */
package cheddar.util;

import cheddar.*;

import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;

/**
 * <!-- begin-user-doc -->
 * The <b>Switch</b> for the model's inheritance hierarchy.
 * It supports the call {@link #doSwitch(EObject) doSwitch(object)}
 * to invoke the <code>caseXXX</code> method for each class of the model,
 * starting with the actual class of the object
 * and proceeding up the inheritance hierarchy
 * until a non-null result is returned,
 * which is the result of the switch.
 * <!-- end-user-doc -->
 * @see cheddar.CheddarPackage
 * @generated
 */
public class CheddarSwitch {
	/**
	 * The cached model package
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	protected static CheddarPackage modelPackage;

	/**
	 * Creates an instance of the switch.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @generated
	 */
	public CheddarSwitch() {
		if (modelPackage == null) {
			modelPackage = CheddarPackage.eINSTANCE;
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	public Object doSwitch(EObject theEObject) {
		return doSwitch(theEObject.eClass(), theEObject);
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected Object doSwitch(EClass theEClass, EObject theEObject) {
		if (theEClass.eContainer() == modelPackage) {
			return doSwitch(theEClass.getClassifierID(), theEObject);
		}
		else {
			List eSuperTypes = theEClass.getESuperTypes();
			return
				eSuperTypes.isEmpty() ?
					defaultCase(theEObject) :
					doSwitch((EClass)eSuperTypes.get(0), theEObject);
		}
	}

	/**
	 * Calls <code>caseXXX</code> for each class of the model until one returns a non null result; it yields that result.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the first non-null result returned by a <code>caseXXX</code> call.
	 * @generated
	 */
	protected Object doSwitch(int classifierID, EObject theEObject) {
		switch (classifierID) {
			case CheddarPackage.DEPENDENCY_ELEMENT: {
				DependencyElement dependencyElement = (DependencyElement)theEObject;
				Object result = caseDependencyElement(dependencyElement);
				if (result == null) result = caseElement(dependencyElement);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.ELEMENT: {
				Element element = (Element)theEObject;
				Object result = caseElement(element);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.PROCESSOR: {
				processor processor = (processor)theEObject;
				Object result = caseprocessor(processor);
				if (result == null) result = caseElement(processor);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.SCHEDULER: {
				scheduler scheduler = (scheduler)theEObject;
				Object result = casescheduler(scheduler);
				if (result == null) result = caseElement(scheduler);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.NETWORK: {
				network network = (network)theEObject;
				Object result = casenetwork(network);
				if (result == null) result = caseElement(network);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.ADDRESS_SPACE: {
				address_space address_space = (address_space)theEObject;
				Object result = caseaddress_space(address_space);
				if (result == null) result = caseElement(address_space);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.TASK: {
				task task = (task)theEObject;
				Object result = casetask(task);
				if (result == null) result = caseDependencyElement(task);
				if (result == null) result = caseElement(task);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.OFFSET: {
				offset offset = (offset)theEObject;
				Object result = caseoffset(offset);
				if (result == null) result = caseElement(offset);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.PARAMETER: {
				parameter parameter = (parameter)theEObject;
				Object result = caseparameter(parameter);
				if (result == null) result = caseElement(parameter);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.EVENT_ANALYZER: {
				event_analyzer event_analyzer = (event_analyzer)theEObject;
				Object result = caseevent_analyzer(event_analyzer);
				if (result == null) result = caseElement(event_analyzer);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.BUFFER: {
				buffer buffer = (buffer)theEObject;
				Object result = casebuffer(buffer);
				if (result == null) result = caseDependencyElement(buffer);
				if (result == null) result = caseElement(buffer);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.BUFFER_USER: {
				buffer_user buffer_user = (buffer_user)theEObject;
				Object result = casebuffer_user(buffer_user);
				if (result == null) result = caseElement(buffer_user);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.DEPENDENCY: {
				dependency dependency = (dependency)theEObject;
				Object result = casedependency(dependency);
				if (result == null) result = caseElement(dependency);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.MESSAGE: {
				message message = (message)theEObject;
				Object result = casemessage(message);
				if (result == null) result = caseDependencyElement(message);
				if (result == null) result = caseElement(message);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.RESOURCE_USER: {
				resource_user resource_user = (resource_user)theEObject;
				Object result = caseresource_user(resource_user);
				if (result == null) result = caseElement(resource_user);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.RESOURCE: {
				resource resource = (resource)theEObject;
				Object result = caseresource(resource);
				if (result == null) result = caseElement(resource);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			case CheddarPackage.ROOT: {
				Root root = (Root)theEObject;
				Object result = caseRoot(root);
				if (result == null) result = defaultCase(theEObject);
				return result;
			}
			default: return defaultCase(theEObject);
		}
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>Dependency Element</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>Dependency Element</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseDependencyElement(DependencyElement object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>Element</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>Element</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseElement(Element object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>processor</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>processor</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseprocessor(processor object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>scheduler</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>scheduler</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casescheduler(scheduler object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>network</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>network</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casenetwork(network object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>address space</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>address space</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseaddress_space(address_space object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>task</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>task</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casetask(task object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>offset</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>offset</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseoffset(offset object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>parameter</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>parameter</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseparameter(parameter object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>event analyzer</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>event analyzer</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseevent_analyzer(event_analyzer object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>buffer</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>buffer</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casebuffer(buffer object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>buffer user</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>buffer user</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casebuffer_user(buffer_user object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>dependency</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>dependency</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casedependency(dependency object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>message</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>message</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object casemessage(message object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>resource user</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>resource user</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseresource_user(resource_user object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>resource</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>resource</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseresource(resource object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>Root</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>Root</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject) doSwitch(EObject)
	 * @generated
	 */
	public Object caseRoot(Root object) {
		return null;
	}

	/**
	 * Returns the result of interpretting the object as an instance of '<em>EObject</em>'.
	 * <!-- begin-user-doc -->
	 * This implementation returns null;
	 * returning a non-null result will terminate the switch, but this is the last case anyway.
	 * <!-- end-user-doc -->
	 * @param object the target of the switch.
	 * @return the result of interpretting the object as an instance of '<em>EObject</em>'.
	 * @see #doSwitch(org.eclipse.emf.ecore.EObject)
	 * @generated
	 */
	public Object defaultCase(EObject object) {
		return null;
	}

} //CheddarSwitch
