/**
 * <copyright>
 * Thales ATL for MARTE with RSA (Copyright (c) THALES 2007 All rights reserved)
 * is free software; you can redistribute it and/or modify
 * it under the terms of the Eclipse Public License as published
 * in http://www.eclipse.org/legal/epl-v10.html
 * Thales MARTE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the Eclipse Public License for more details.
 * </copyright>
 *
 * $Id: MARTEImplementation_RSA.java,v 1.3 2007/11/21 13:59:59 emaes Exp $
 */
package com.thalesgroup.atl.marte.rsa;

/**
 * RSA MARTE Implementation specificities
 * @author Eric MAES
 */

import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.OpaqueExpression;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.ValueSpecification;

import com.cea.nfp.parsers.modelgenerator.IModelFacade;
import com.thalesgroup.atl.exception.ATLException;
import com.thalesgroup.atl.marte.MARTEFacilities;
import com.thalesgroup.atl.marte.MARTEImplementation;
import com.thalesgroup.nfp.rsa.facade.RSAModelFacade;

public class MARTEImplementation_RSA extends MARTEImplementation {

	
	public MARTEImplementation_RSA() {
		super("/libraries/MARTE4RSA.asm", new MARTE4RSA(), new ATLConstantsLoader_MARTE4RSA());
	}
	
	public boolean isRSA() {
		return (true);
	}
	
	public boolean isPAPYRUS() {
		return (false);
	}
	
	public String getLibraryPath() throws ATLException {
		return (this.MARTE_ATL_LIBRARY_PATH);
	}
	
	public IModelFacade getFacade(Element element) throws ATLException {
// TODO New version of VSL Parser
		return RSAModelFacade.getInstance(element.getModel(), element);
// TODO	To keep for old VSL Parser version
//		return (new RSAModelFacade(element.getModel(),element));
	}
	
	/**
	 * @depracated
	 */
	public ValueSpecification getPropertyValue(Element element, Property property, Object propertyValue)
		throws ATLException
	{
		ValueSpecification retour = null;
		if (propertyValue instanceof DynamicEObjectImpl) {
			DynamicEObjectImpl propertyValueDynamicEObjectImpl = (DynamicEObjectImpl) propertyValue;
			EClass propertyValueEClass = propertyValueDynamicEObjectImpl.eClass();
			EStructuralFeature vslEStructuralFeature = propertyValueEClass.getEStructuralFeature("value");
			Object vs = propertyValueDynamicEObjectImpl.eGet(vslEStructuralFeature);
			retour = getPropertyValue(element, property, vs);
		}
		if (propertyValue instanceof Property) {
			Property p = (Property) propertyValue;
			retour = getPropertyValue(element,property,p.getDefaultValue());
		}
		if (propertyValue instanceof OpaqueExpression) {
			OpaqueExpression opaqueExpressionProperty = (OpaqueExpression) propertyValue;
			retour = getPropertyValue(element, property, opaqueExpressionProperty.getBodies());
		}
		if (propertyValue instanceof String) {
			String stringProperty = (String) propertyValue;
			retour = MARTEFacilities.marteGetValueSpecification(element,property,stringProperty);
		}
		if (propertyValue instanceof ValueSpecification) {
			ValueSpecification valueSpecificationProperty = (ValueSpecification) propertyValue;
			retour = valueSpecificationProperty;
		}
		// TODO Only lists with one element are considered
		if (propertyValue instanceof EList) {
			EList propertyValues = (EList) propertyValue;
			Iterator pV = propertyValues.iterator();
			if (propertyValues.size()>1) {
				String message = "Lists with more than one element are not taken into account";
				throw new ATLException(message);
			} else {
				if (pV.hasNext()) {
					Object pVValue = pV.next();
					retour = getPropertyValue(element,property,pVValue);
				}
			}
		}
		if (retour==null) {
			String message = "Error getting the Property value of "+propertyValue;
			throw new ATLException(message);
		}
		return(retour);
	}
	

	public ValueSpecification getPropertyValue(Element element, Property property, Object propertyValue, ArrayList<String> attribList)
		throws ATLException
	{
		ValueSpecification retour = null;
		if (propertyValue instanceof DynamicEObjectImpl) {
			DynamicEObjectImpl propertyValueDynamicEObjectImpl = (DynamicEObjectImpl) propertyValue;
			EClass propertyValueEClass = propertyValueDynamicEObjectImpl.eClass();
			EStructuralFeature vslEStructuralFeature = propertyValueEClass.getEStructuralFeature("value");
			Object vs = propertyValueDynamicEObjectImpl.eGet(vslEStructuralFeature);
			retour = getPropertyValue(element, property, vs,attribList);
		} else if (propertyValue instanceof Property) {
			Property p = (Property) propertyValue;
			retour = getPropertyValue(element,p,p.getDefaultValue(),attribList);
		} else if (propertyValue instanceof OpaqueExpression) {
			OpaqueExpression opaqueExpressionProperty = (OpaqueExpression) propertyValue;
			retour = getPropertyValue(element, property, opaqueExpressionProperty.getBodies(),attribList);
		} else if (propertyValue instanceof ValueSpecification) {
				ValueSpecification valueSpecificationProperty = (ValueSpecification) propertyValue;
				retour = valueSpecificationProperty;
		} else if (propertyValue instanceof String) {
			String stringProperty = (String) propertyValue;
			retour = MARTEFacilities.marteGetValueSpecification(element,property,stringProperty);
		} else if (propertyValue instanceof EList) {
			EList propertyValues = (EList) propertyValue;
			Iterator pV = propertyValues.iterator();
			while (pV.hasNext()) {
				Object pVValue = pV.next();
				if (pVValue instanceof NamedElement) {
					NamedElement pVValueElement = (NamedElement) pVValue;
					if (attribList.contains(pVValueElement.getName())) {
						attribList.remove(pVValueElement.getName());
						retour = getPropertyValue(element,property,pVValueElement,attribList);
					}
				} else {
					retour = getPropertyValue(element,property,pVValue,attribList);
				}
			}
		}
		return(retour);
	}

}
